<?php
/**
 * @brief		Member Sync
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Invite System
 * @since		10 Apr 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\invite\extensions\core\MemberSync;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Member Sync
 */
class _Invite
{
	/**
	 * Member is merged with another member
	 *
	 * @param	\IPS\Member	$member		Member being kept
	 * @param	\IPS\Member	$member2	Member being removed
	 * @return	void
	 */
	public function onMerge( $member, $member2 )
	{
		\IPS\Db::i()->update( 'invite_invites', array( 'invite_sender_id' => $member->member_id ), array( 'invite_sender_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'invite_invites', array( 'invite_conv_member_id' => $member->member_id ), array( 'invite_conv_member_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'invite_members', array( 'is_member_id' => $member->member_id ), array( 'is_member_id=?', $member2->member_id ), array(), NULL, \IPS\Db::IGNORE );
	}

	/**
	 * Member is deleted
	 *
	 * @param	$member	\IPS\Member	The member
	 * @return	void
	 */
	public function onDelete( $member )
	{
		\IPS\Db::i()->delete( 'invite_invites', array( 'invite_sender_id=?', $member->member_id ) );
		\IPS\Db::i()->delete( 'invite_invites', array( 'invite_conv_member_id=?', $member->member_id ) );
		\IPS\Db::i()->delete( 'invite_members', array( 'is_member_id=?', $member->member_id ) );
	}
}