<?php
/**
 * @brief		Members Shop Extension: Invite
 * @author		<a href='https://www.ipsdev.org'>IPSDev</a>
 * @copyright	(c) ipsdev.org
 * @package		Members Shop
 * @subpackage	Invite System
 * @since		07 Dec 2022
**/

namespace IPS\invite\extensions\membersshop\Items;

/* To prevent PHP errors (extending class does not exist) revealing path */
if( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _Invite
{
	public static function isActive()
    {
        return TRUE;
    }
	
	/** 
	  *	This function is for the items list in the add items page of the ACP
	  *	These will be added to the array list for the select $form settings, you need to edit the 1234 in the $options[1234] to a unique id
	  *	The best way to create a unique id will be to use the unix timestamp of your current time, There is more chance of winning the lottery then 2 or more devs creating a item at the same time
	  *	You can add multiple items here just make sure each item has a unique id
	  * If you are adding multiple items just +1 the unix timestamp so if it ends 69 change it to 70
	  * You need to add a font awesome icon for the setting display just add the icons name just use the same icon you want to use for the item
	**/
	public function itemOptions( &$options, &$fontAwesome )
    {
		$options[8765] = 'is_ms_buy_invitation';
		$fontAwesome[8765] = 'fa-ticket';
	}
	
	/** 
	  *	This function is for saving the values of the $form from the settings above
	  * These are the default values what will get added when created
	  * Once created the user will be directed to edit it anyway so it don't really matter what default values you put as long as it is is acceptable
	  *	If you have added multiple $options from the itemOptions function then you will need to add elseif statements below for them
	  * Any NULL values are not required but can be used in the processRedeem function below to pull data or values needed to run your script
	**/
	public function itemFormSubmitted( $values, $item )
	{
		if( $values['ms_add_item_to_shop'] == 8765 )
		{
			$item->name = \IPS\Member::loggedIn()->language()->get( 'is_ms_buy_invitation' ); // Change the language string to match the one in the itemOptions function
			$item->price = 2500;
			$item->open = 1;
			$item->purchases = 0;
			$item->desc = 'This will allow members to buy invitations to be used in the Invite System.'; // Enter a brief description to give the admin some information of what the item does
			$item->icon = 'fa-ticket';
			$item->instock = -1;
			$item->cat_id = \IPS\Request::i()->parent;
			$item->submitted = time();
			$item->item_id = $values['ms_add_item_to_shop'];
			
			/* The 4 values listed below are not required but might be needed for additional information to be passed onto the processRedeem function */
			$item->maxRand = NULL; // INT 255 column to use for maybe random amounts for the minimum amount 
			$item->minRand = NULL; // INT 255 column to use for maybe random amounts for the maximum amount 
			$item->extra_field = NULL; // INT 255 colums used to store any other numbers needed to be stored
			$item->extra_field_text = NULL; // VCHAR 999 Column to add additional text for the items
			
			$item->save();
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=items&subnode=1&do=form&id='.$item->id.'' ), 'saved' );
		}
	}
	
	/** 
	  *	This function is for allowing the users to edit the values of the items, Any required settings will already be shown on the edit page
	  *	If you have added multiple $options from the itemOptions function then you will need to add elseif statements below for them
	  * You only need to use this function if you are using any of the non required columns listed in the itemFormSubmitted function
	**/
	public function editFormSettings( &$form, $item )
    {
		/*
		if( $item->item_id == 1234 ) // Edit the 1234 to match your unique id in the itemOptions function
		{
	    	$form->add( new \IPS\Helpers\Form\Number( 'my_settings', $item->minRand, TRUE, array( 'min' => 0 ), NULL, NULL, NULL, 'my_settings' ) );
			$form->add( new \IPS\Helpers\Form\Number( 'my_settings_two', $item->maxRand, TRUE, array( 'min' => 0 ), NULL, NULL, NULL, 'my_settings_two' ) );
			$form->add( new \IPS\Helpers\Form\Number( 'my_settings_three', $item->extra_field, TRUE, array( 'min' => 0 ), NULL, NULL, NULL, 'my_settings_three' ) );
			$form->add( new \IPS\Helpers\Form\Text( 'my_settings_four', $item->extra_field_text, TRUE, array( 'min' => 0 ), NULL, NULL, NULL, 'my_settings_four' ) );
		}
		*/
	}
	
	/** 
	  *	This function is for when a admin generates a purchase for a user, Any required columns will process automatically
	  *	If you have added multiple $options from the itemOptions function then you will need to add elseif statements below for them
	  * The $values are your settings values in the editFormSettings function
	  * You only need to use this function if you are using any of the non required columns listed in the itemFormSubmitted function
	**/
	public function processGenerateForm( $values, $purchase, $item )
    {
		/*
		if( $item->item_id == 1234 ) // Edit the 1234 to match your unique id in the itemOptions function
		{
			$purchase->minRand = $values['my_settings']; 
			$purchase->maxRand = $values['my_settings_two'];
			$purchase->extra_field = $values['my_settings_three'];
			$purchase->extra_field_text = $values['my_settings_four'];
			$purchase->save();
		}
		*/
	}
	
	/** 
	  *	This function is for saving a edited item, Any required columns will save automatically, the $values are from your settings in the editFormSettings function
	  *	If you have added multiple $options from the itemOptions function then you will need to add elseif statements below for them
	  * You only need to use this function if you are using any of the non required columns listed in the itemFormSubmitted function
	**/
	public function processEditForm( $item, $values )
    {		
		/*
		if( $item->item_id == 1234 ) // Edit the 1234 to match your unique id in the itemOptions function
		{
			$item->minRand = $values['my_settings'];
			$item->maxRand = $values['my_settings_two'];
			$item->extra_field = $values['my_settings_three'];
			$item->extra_field_text = $values['my_settings_four'];
			$item->save();
		}
		*/
	}

	/** 
	  *	This function is for when a user redeems the item we are going to redirect them to the processRedeem function
	  * If you have added multiple $options from the itemOptions function then you will need to add elseif statements below for them
	**/
	public function redeemItem( $item_id, $getItem, $member, $boughtAndUsed, $categoryURL )
	{		
		if( $item_id == 8765 ) { return $this->processRedeem( $getItem, $member, $boughtAndUsed, $categoryURL ); }
	}

	/** 
	  *	This function is the actual process of the redemption of the items, the $getItem will call all the data stored in the database
	  *	If you have added multiple $options from the itemOptions function then you will need to add elseif statements below for them
	**/
	public function processRedeem( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		if( $getItem->item_id == 8765 )
		{
			$user = \IPS\invite\Member::getMember( $member );
			$user->invites_remaining += 1;
			$user->save();

			$getItem->redeemed = 1;
			$getItem->save();

			$logsACP = \IPS\Member::loggedIn()->language()->get('is_ms_acp_log_invitation_purchase');
			$member->removePoints( $getItem->price, $logsACP );

			$outputLang = \IPS\Member::loggedIn()->language()->addToStack( 'is_ms_buy_invitation_done' );

			if( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}

				\IPS\Output::i()->redirect( $redirectURL, $outputLang );
			}
		}
	}
}