//<?php

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

class invite_hook_is_RegisterScreen extends _HOOK_CLASS_
{
	/**
	 * Build Registration Form
	 *
	 * @param	array|NULL	$postBeforeRegister	The row from core_post_before_registering if applicable
	 * @return	\IPS\Helpers\Form
	 */
	public static function buildRegistrationForm( $postBeforeRegister = NULL )
	{
		try
		{
			try
			{
				try
				{
					try
					{
						try
						{
							$form = parent::buildRegistrationForm( $postBeforeRegister );
					
							if( \IPS\Settings::i()->is_on )
							{
								if( \IPS\invite\Campaign::canUseCampaign() )
								{
									$form->add( new \IPS\Helpers\Form\Radio( 'invite_type', 'invite_type_is', TRUE, array(
										'options' => array(
											'invite_type_is' 	=> 'invite_type_is',
											'invite_type_cp'	=> 'invite_type_cp',
										),
										'toggles'	=> array(
											'invite_type_is'	=> array( 'invite_code' ),
											'invite_type_cp'	=> array( 'invite_campaign_code' )
										)
									), NULL, NULL, NULL, 'invite_type' ), 'password_confirm' );
								}
					
								$after = \IPS\invite\Campaign::canUseCampaign() ? 'invite_type' : 'password_confirm';
					
								$inviteRequired = \IPS\Settings::i()->is_requireinvite ? 'invite_code_req' : 'invite_code_opt';
								$required 		= \IPS\Settings::i()->is_requireinvite ? TRUE : FALSE;
								\IPS\Member::loggedIn()->language()->words['invite_code_desc'] = \IPS\Member::loggedIn()->language()->addToStack( $inviteRequired );
					
								$form->add( new \IPS\Helpers\Form\Text( 'invite_code', NULL, $required, array( 'maxLength' => 32 ), function( $val )
								{
									try
									{
										if( $val )
										{
											if( strlen( $val ) < 32 )
											{
												throw new \DomainException( 'invite_does_not_exist' );
											}
					
											try
											{
												$invitation = \IPS\invite\Invitation::load( $val, 'invite_code' );
											}
											catch( \OutOfRangeException $ex )
											{
													throw new \DomainException( 'invite_does_not_exist' );
											}
					
											if( \IPS\Settings::i()->is_requireinvite )
											{
												if( $invitation->status == 1 )
												{
													throw new \DomainException( 'invite_already_in_use' );
												}
					
												if( $invitation->status == 2 )
												{
													throw new \DomainException( 'invite_already_expired' );
												}
											}
											else
											{
												if( !$val )
												{
													return true;
												}
					
												if( $invitation->status == 1 )
												{
													throw new \DomainException( 'invite_already_in_use' );
												}
					
												if( $invitation->status == 2 )
												{
													throw new \DomainException( 'invite_already_expired' );
													}
											}
										}
									}
									catch( \OutOfRangeException $e )
									{
										return true;
									}
								}, NULL, NULL, 'invite_code' ), $after );
					
								if( \IPS\invite\Campaign::canUseCampaign() )
								{
									$form->add( new \IPS\Helpers\Form\Text( 'invite_campaign_code', NULL, TRUE, array( 'minLength' => 4, 'maxLength' => 32 ), function( $val )
									{
										try
										{
											try
											{
												$campaign = \IPS\invite\Campaign::getCampaignByCode( $val );
											}
											catch( \UnderflowException $ex )
											{
												throw new \DomainException( 'campaign_does_not_exist' );
											}
					
											if( $campaign->max_new_members != -1 AND $campaign->getNumberOfMembersInCampaign() >= $campaign->max_new_members )
											{
												throw new \DomainException( 'campaign_max_members_reached' );
										}
						
											$now	 		= \IPS\DateTime::ts( time() );
											$campaignDays 	= \IPS\DateTime::create()->sub( new \DateInterval( 'P' . $campaign->duration . 'D' ) );
											$timeLeft 		= $now->diff( $campaignDays );
											if( $timeLeft->days > $campaign->duration )
											{
												throw new \DomainException( 'campaign_max_date_limit' );
											}
										}
										catch( \OutOfRangeException $e )
										{
											return true;
										}
									}, NULL, NULL, 'invite_campaign_code' ), $after );
								}
							}
					
							return $form;
						}
						catch ( \Error | \RuntimeException $e )
						{
							if ( method_exists( get_parent_class(), __FUNCTION__ ) )
							{
								return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
							}
							else
							{
								throw $e;
							}
						}
					}
					catch ( \Error | \RuntimeException $e )
					{
						if ( method_exists( get_parent_class(), __FUNCTION__ ) )
						{
							return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
						}
						else
						{
							throw $e;
						}
					}
				}
				catch ( \Error | \RuntimeException $e )
				{
					if ( method_exists( get_parent_class(), __FUNCTION__ ) )
					{
						return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
					}
					else
					{
						throw $e;
					}
				}
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}

	/**
	 * Create Member
	 *
	 * @param	array 				$values   		    Values from form
	 * @param	array				$profileFields		Profile field values from registration
	 * @param	array|NULL			$postBeforeRegister	The row from core_post_before_registering if applicable
	 * @param	\IPS\Helpers\Form	$form				The form object
	 * @return  \IPS\Member
	 */
	public static function _createMember( $values, $profileFields, $postBeforeRegister, &$form )
	{
		try
		{
			try
			{
				try
				{
					try
					{
						try
						{
							if( \IPS\Settings::i()->is_on )
							{
								if( !isset( $values['invite_type'] ) OR isset( $values['invite_type'] ) AND $values['invite_type'] == 'invite_type_is' AND \IPS\Settings::i()->is_requireinvite AND $values['invite_code'] )
								{
									if( $values['invite_code'] === "0" )
									{
										\IPS\Output::i()->error( 'invite_does_not_exist', 'INVITE SYSTEM - CREATE ACCOUNT/2', 403, '' );
									}
					
									if( $values['invite_code'] )
									{
										try
										{
											$invitation = \IPS\invite\Invitation::load( $values['invite_code'], 'invite_code' );
										}
										catch( \OutOfRangeException $ex )
										{
											if( \IPS\Settings::i()->is_requireinvite )
											{
												throw new \DomainException( 'invite_does_not_exist' );
											}
										}
					
										if( \IPS\Settings::i()->is_emailfrominvite )
										{
											if( $values['email_address'] != $invitation->invited_email )
											{
												\IPS\Output::i()->error( 'is_error_notsameemails', 'INVITE SYSTEM - CREATE ACCOUNT/1', 403, '' );
											}
										}
									}
								}
							}
					
							$member = parent::_createMember( $values, $profileFields, $postBeforeRegister, $form );
					
							if( \IPS\Settings::i()->is_on )
							{
								if( !isset( $values['invite_type'] ) OR isset( $values['invite_type'] ) AND $values['invite_type'] == 'invite_type_is' )
								{
									if( $values['invite_code'] )
									{
										try
										{
											$invitation = \IPS\invite\Invitation::load( $values['invite_code'], 'invite_code' );
										}
										catch( \OutOfRangeException $ex )
										{
											throw new \DomainException( 'invite_does_not_exist' );
										}
					
										if( \IPS\Settings::i()->is_emailfrominvite )
										{
											if( $values['email_address'] != $invitation->invited_email )
											{
												\IPS\Output::i()->error( 'is_error_notsameemails', 'INVITE SYSTEM - CREATE ACCOUNT/2', 403, '' );
												return $member;
											}
										}
					
										if( \IPS\Settings::i()->is_registrationgroup_toggle )
										{
											$member->member_group_id = \IPS\Settings::i()->is_registrationgroup;
										}
										$member->save();
					
										$user = new \IPS\invite\Member;
										$user->member_id 	 = $member->member_id;
										$user->invite_offset = 0;
										$user->invited_by 	 = $invitation->sender_id;
										$user->revoke_access = 0;
					
										if( \IPS\Settings::i()->is_registration_earninvitations AND \IPS\Settings::i()->is_registration_earninvitations_nr > 0 )
										{
											$user->invites_remaining = \IPS\Settings::i()->is_registration_earninvitations_nr;
										}
										$user->save();
					
										$inviter = \IPS\Member::load( $invitation->sender_id );
										if( $inviter->member_id AND \IPS\invite\Invite::canUsePoints() AND \IPS\Settings::i()->is_mshop AND \IPS\Settings::i()->is_mshop_points > 0 )
										{
											$log = \IPS\Member::loggedIn()->language()->addToStack( 'is_points_added', FALSE, array( 'sprintf' => ( array( \IPS\Settings::i()->ms_points_name, $member->url(), $member->name ) ) ) );
											\IPS\Member::loggedIn()->language()->parseOutputForDisplay( $log );
											$inviter->addPoints( \IPS\Settings::i()->is_mshop_points, $log );
										}
					
										if( $inviter->member_id )
										{
											$notification = new \IPS\Notification( \IPS\Application::load( 'invite' ), 'account_converted', $member, array( $member->name, $member->url() ) );
											$notification->recipients->attach( $inviter );
											$notification->send();
										}
						
										$invitation->status 		 = 1;
										$invitation->conv_member_id  = $member->member_id;
										$invitation->conv_date		 = time();
										$invitation->expiration_date = 0;
										$invitation->save();
										$data = array(
											'inviter' 	=> $inviter,
											'inviteer' 	=> $member
										);
										\IPS\Api\Webhook::fire( 'is_account_registered', $data );
										$inviter->achievementAction( 'invite', 'invitation', array( 'user' => $inviter ) );
									}
								}
					
								if( isset( $values['invite_type'] ) AND $values['invite_type'] == 'invite_type_cp' )
								{
									$campaign = \IPS\invite\Campaign::getCampaignByCode( $values['invite_campaign_code'] );
									if( $campaign->id )
									{
										$toInsert = array(
											'cpg_mid_mid'		=> $member->member_id,
											'cpg_mid_campain'	=> $campaign->id,
											'cpg_mid_date'		=> time()
										);
										\IPS\Db::i()->insert( 'invite_campaigns_members', $toInsert, TRUE );
					
										$campaign->converted_members += 1;
										$campaign->save();
									}
								}
							}
						
					        return $member;
						}
						catch ( \Error | \RuntimeException $e )
						{
							if ( method_exists( get_parent_class(), __FUNCTION__ ) )
							{
								return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
							}
							else
							{
								throw $e;
							}
						}
					}
					catch ( \Error | \RuntimeException $e )
					{
						if ( method_exists( get_parent_class(), __FUNCTION__ ) )
						{
							return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
						}
						else
						{
							throw $e;
						}
					}
				}
				catch ( \Error | \RuntimeException $e )
				{
					if ( method_exists( get_parent_class(), __FUNCTION__ ) )
					{
						return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
					}
					else
					{
						throw $e;
					}
				}
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
}