<?php

namespace IPS\invite\modules\admin\invite;

/* To prevent PHP errors (extending class does not exist) revealing path */
if( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _invite extends \IPS\Dispatcher\Controller
{	
	public static $csrfProtected = TRUE;

	protected function manage()
	{
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('menu__invite_invite_invite');

		$table = new \IPS\Helpers\Table\Db( 'invite_invites', \IPS\Http\Url::internal( 'app=invite&module=invite&controller=invite' ) );
		$table->joins = array(
			array(
				'select'	=> 'core_members.*',
				'from'		=> 'core_members',
				'where'		=> 'core_members.member_id=invite_invites.invite_sender_id'
			)
		);

		$table->limit = 10;
		$table->tableTemplate  = array( \IPS\Theme::i()->getTemplate( 'view', 'invite', 'admin' ), 'inviteTable' );
		$table->rowsTemplate   = array( \IPS\Theme::i()->getTemplate( 'view', 'invite', 'admin' ), 'inviteRows' );

        $table->include 	= array( 'invite_id', 'photo', 'invite_sender_id', 'invite_code', 'invite_date', 'invite_invited_name', 'invite_invited_email', 'invite_status', 'member_group_id' );
		$table->mainColumn 	= 'invite_sender_id';
		$table->noSort		= array( 'photo' );

		$table->filters = array(
			'invites_filter_pending'	=> 'invite_status = 0',
			'invites_filter_converted'	=> 'invite_status = 1',
			'invites_filter_expired'	=> 'invite_status = 2',
		);

		$filterOptions = array(
			'0'	=> 'invites_filter_pending',
			'1'	=> 'invites_filter_converted',
			'2'	=> 'invites_filter_expired',
		);

		$table->advancedSearch = array(
			'invite_sender_id'		=> \IPS\Helpers\Table\SEARCH_MEMBER,
			'invite_invited_name'	=> \IPS\Helpers\Table\SEARCH_CONTAINS_TEXT,
			'invite_invited_email'	=> \IPS\Helpers\Table\SEARCH_CONTAINS_TEXT,
			'invite_status'			=> array( \IPS\Helpers\Table\SEARCH_SELECT, array( 'options' => $filterOptions ) ),
		);

		$table->advancedSearchCallback = function( $table, $values )
		{
			switch ( $values['invite_status'] )
			{
				case 0:
					$table->where[] = array( 'invite_status=?', 0 );
					break;
				case 1:
					$table->where[] = array( 'invite_status=?', 1 );
					break;
				case 2:
					$table->where[] = array( 'invite_status=?', 2 );
					break;
			}
		};

        $table->parsers = array(
            'invite_sender_id'	=> function( $val, $row )
            {
				return "<a target='_blank' href='" . \IPS\Http\Url::internal( 'app=core&module=members&controller=members&do=view&id=' ) . $row['invite_sender_id'] . "'>" . $row['name'] . "</a>";
            },
			'member_group_id' => function( $val, $row )
			{
				return \IPS\Member\Group::load( $row['member_group_id'] )->formattedName;
			},
            'photo' => function( $val, $row )
            {
                return \IPS\Theme::i()->getTemplate( 'global', 'core' )->userPhoto( \IPS\Member::constructFromData( $row ), 'tiny' );
            },
            'invite_invited_name' => function( $val, $row )
            {
                return $row['invite_invited_name'];
            },
            'invite_invited_email' => function( $val, $row )
            {
                return $row['invite_invited_email'];
            },
            'invite_date' => function( $val, $row )
            {
                return \IPS\DateTime::ts( $val )->relative();
            },
            'invite_status' => function( $val, $row )
            {
				switch( $row['invite_status'] )
				{
					case 0:
						$text = 'invite_status_pending';
					break;
					case 1:
						$text = 'invite_status_converted';
					break;
					case 2:
						$text = 'invite_status_expired';
					break;
				}

				return \IPS\Member::loggedIn()->language()->addToStack( $text );
            },
        );

		\IPS\Output::i()->sidebar['actions']['add'] = array(
			'primary'	=> true,
			'icon'		=> 'plus',
			'link'		=> \IPS\Http\Url::internal( 'app=invite&module=invite&controller=invite&do=add' )->csrf(),
			'title'		=> 'send_invitation',
			'data'		=> array( 'ipsDialog' => '', 'ipsDialog-size' => 'medium', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('send_invitation') )
		);

		if( \IPS\Member::loggedIn()->hasAcpRestriction( 'invite', 'invite', 'invite_batch' ) )
		{
			\IPS\Output::i()->sidebar['actions']['batch'] = array(
				'icon'	=> 'plus',
				'link'	=> \IPS\Http\Url::internal( 'app=invite&module=invite&controller=invite&do=batch' )->csrf(),
				'title'	=> 'is_batch_invitations',
				'data'	=> array( 'ipsDialog' => '', 'ipsDialog-size' => 'medium', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('is_batch_invitations') )
			);
		}

		if( \IPS\Member::loggedIn()->hasAcpRestriction( 'invite', 'invite', 'invite_export' ) AND \IPS\Db::i()->select( 'COUNT(*)', 'invite_invites' )->first() )
		{
			\IPS\Output::i()->sidebar['actions']['download'] = array(
				'icon'	=> 'download',
				'link'	=> \IPS\Http\Url::internal( 'app=invite&module=invite&controller=invite&do=download' ),
				'title'	=> 'is_download_invitations'
			);
		}

		if( \IPS\Db::i()->select( 'COUNT(*)', 'invite_invites', array( 'invite_status=?', 2 ) )->first() )
		{
			$lang = \IPS\Settings::i()->is_restoreinvitation ? 'delete_all_expired_willrestore' : 'delete_all_expired_wontrestore';
			\IPS\Output::i()->sidebar['actions']['delete'] = array(
				'icon'	=> 'trash',
				'link'	=> \IPS\Http\Url::internal( 'app=invite&module=invite&controller=invite&do=deleteAll' )->csrf(),
				'title'	=> 'delete_all_expired',
				'data'	=> array( 'delete' => '', 'delete-warning' => \IPS\Member::loggedIn()->language()->addToStack( $lang ) )
			);
		}

		$table->rowButtons = function( $row )
		{
			$return = array();
			if( $row['invite_status'] == 1 )
			{
				$return = array( 'view' => array(
					'icon'		=> 'user',
					'title'		=> 'view_member',
					'link'		=> \IPS\Http\Url::internal( "app=invite&module=invite&controller=invite&do=member&id={$row['invite_conv_member_id']}" )->csrf(),
					'data'		=> array( 'ipsDialog' => '', 'ipsDialog-size' => "narrow", 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('view_member') ),
					'hotkey'	=> 'v'
				) );
			}

			if( $row['invite_status'] != 1 AND \IPS\Member::loggedIn()->hasAcpRestriction( 'invite', 'invite', 'invite_delete' ) )
			{
				$return['delete'] = array(
					'icon'		=> 'trash-o',
					'title'		=> 'delete_invite',
					'link'		=> \IPS\Http\Url::internal( "app=invite&module=invite&controller=invite&do=delete&id={$row['invite_id']}" )->csrf(),
					'data'		=> array( 'delete' => '' ),
					'hotkey'	=> 'd'
				);

				if( \IPS\Member::loggedIn()->hasAcpRestriction( 'invite', 'invite', 'invite_resend' ) )
				{
					$return['resend'] = array(
						'icon'		=> 'envelope',
						'title'		=> 'resend_invite',
						'link'		=> \IPS\Http\Url::internal( "app=invite&module=invite&controller=invite&do=resend&id={$row['invite_id']}" )->csrf(),
						'data'		=> array( 'confirm' => '' ),
						'hotkey'	=> 'r'
					);
				}
			}

			return $return;
		};

		$table->sortBy = $table->sortBy ?: 'invite_id';
		$table->sortDirection = $table->sortDirection ?: 'desc';

		\IPS\Output::i()->output = $table;
	}

	/**
	 * Delete invite
	 *
	 * @return	void
	 */
	protected function delete()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'invite_delete' );

		\IPS\Request::i()->confirmedDelete();
		\IPS\Session::i()->csrfCheck();

		if( !\IPS\Request::i()->id )
		{
			\IPS\Output::i()->error( 'node_error', 'INVITE SYSTEM - DELETE INVITATION/1', 404, '' );
		}

		try
		{
			$invitation = \IPS\invite\Invitation::load( \IPS\Request::i()->id );
		}
		catch( \Exception $e )
		{
			\IPS\Output::i()->error( 'invite_does_not_exist', 'INVITE SYSTEM - DELETE INVITATION/2', 404, '' );
		}

		$sender = $invitation->sender_id;

		$invitation->delete();

		if( \IPS\Settings::i()->is_restoreinvitation )
		{
			$inviter 	= \IPS\Member::load( $sender );
			$user 		= \IPS\invite\Member::getMember( $inviter );
			$groupData	= \IPS\Data\Store::i()->inviteGroups[ $user->member_group_id ];

			if( !$groupData['is_unlimited'] )
			{
				$user->invites_remaining += 1;
				$user->save();
			}
		}

		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=invite&module=invite&controller=invite' ), 'invite_deleted' );
	}

	/**
	 * Delete all expired invitations
	 *
	 * @return	void
	 */
	protected function deleteAll()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'invite_delete' );

		\IPS\Request::i()->confirmedDelete();
		\IPS\Session::i()->csrfCheck();

		if( \IPS\Settings::i()->is_restoreinvitation )
		{
			foreach( \IPS\Db::i()->select( '*', 'invite_invites',  array( 'invite_status=?', 2 ) ) as $invite )
			{
				$inviter 	= \IPS\Member::load( $invite['invite_sender_id'] );
				$user 		= \IPS\invite\Member::getMember( $inviter );
				$groupData	= \IPS\Data\Store::i()->inviteGroups[ $user->member_group_id ];
				
				if( !$groupData['is_unlimited'] )
				{
					$user->invites_remaining += 1;
					$user->save();
				}

				\IPS\Db::i()->delete( 'invite_invites', array( 'invite_id=?', $invite['invite_id'] ) );
			}
		}
		else
		{
			\IPS\Db::i()->delete( 'invite_invites', array( 'invite_status=?', 2 ) );
		}

		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=invite&module=invite&controller=invite' ), 'expired_invites_deleted' );
	}

	/**
	 * Add/Send invite
	 *
	 * @return	void
	 */
	protected function add()
	{
		\IPS\Session::i()->csrfCheck();

		$form = new \IPS\Helpers\Form( 'form', 'send_invitation' );
		
		$form->add( new \IPS\Helpers\Form\Member( 'invite_sender_id', NULL, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Text( 'invite_invited_name', NULL, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Email( 'invite_invited_email', NULL, TRUE, array(), function( $val ){
			
			/* Check email exists */
			foreach( \IPS\Login::methods( TRUE ) as $k => $handler )
			{
				if( $handler->emailIsInUse( $val ) === TRUE )
				{
					throw new \LogicException( 'email_already_in_use' );
					break;
				}
			}

			/* Check if the email was already invited */
			try
			{
				$invite = \IPS\Db::i()->select( '*', 'invite_invites', array( 'invite_invited_email=?', $val ) )->first();
				if( $invite['invite_invited_email'] )
				{
					throw new \LogicException( 'invite_email_already_invited' );
				}
			}
			catch( \UnderflowException $e ){}
		}) );

		if( $values = $form->values() )
		{
			$newDate = 0;
			if( \IPS\Settings::i()->is_expireinvite > 0 )
			{
				$expire  = new \IPS\DateTime;
				$expire->add( new \DateInterval( 'P' . \IPS\Settings::i()->is_expireinvite . 'D' ) );
				$newDate = $expire->getTimestamp();
			}

			$invitationCode = md5( uniqid( microtime(), true ) );
			$invitation = new \IPS\invite\Invitation;
			$invitation->sender_id 		 = $values['invite_sender_id']->member_id;
			$invitation->code			 = $invitationCode;
			$invitation->date 			 = time();
			$invitation->invited_name	 = $values['invite_invited_name'];
			$invitation->invited_email	 = $values['invite_invited_email'];
			$invitation->status			 = 0;
			$invitation->expiration_date = $newDate;
			$invitation->save();

			/* Update # of Invites of the sender */
			$sender 	= \IPS\Member::load( $values['invite_sender_id']->member_id );
			$user 		= \IPS\invite\Member::getMember( $sender );
			$groupData	= \IPS\Data\Store::i()->inviteGroups[ $user->member_group_id ];

			if( !$groupData['is_unlimited'] )
			{
				$user->invites_remaining -= 1;
				$user->save();
			}

			/* Send the invitation */
			\IPS\invite\Invite::sendInvitation( $values['invite_invited_email'], $values['invite_invited_name'], $invitationCode, \IPS\Member::loggedIn()->email, \IPS\Member::loggedIn()->name, $newDate );

			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=invite&module=invite&controller=invite" ), 'invite_sent' );
		}

		\IPS\Output::i()->output = $form;
	}

	/**
	 * Veiw converted invite
	 *
	 * @return	void
	 */
	protected function member()
	{
		\IPS\Session::i()->csrfCheck();

		if( !\IPS\Request::i()->id )
		{
			\IPS\Output::i()->error( 'node_error', 'INVITE SYSTEM - MEMBER DATA/1', 404, '' );
		}

		$member = \IPS\Member::load( \IPS\Request::i()->id );
		$user 	= \IPS\invite\Member::getMember( $member );

		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'view', 'invite', 'admin' )->member( $member, $user );
	}

	protected function resend()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'invite_resend' );

		\IPS\Session::i()->csrfCheck();

		if( !\IPS\Request::i()->id )
		{
			\IPS\Output::i()->error( 'node_error', 'INVITE SYSTEM - RESEND INVITATION/1', 404, '' );
		}

		try
		{
			$invitation = \IPS\invite\Invitation::load( \IPS\Request::i()->id );
		}
		catch( \Exception $e )
		{
			\IPS\Output::i()->error( 'invite_does_not_exist', 'INVITE SYSTEM - RESEND INVITATION/2', 404, '' );
		}

		\IPS\invite\Invite::sendInvitation( $invitation->invited_email, $invitation->invited_name, $invitation->code, \IPS\Member::loggedIn()->email, \IPS\Member::loggedIn()->name, $invitation->expiration_date );

		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=invite&module=invite&controller=invite" ), 'invite_resent' );
	}

	/**
	 * Add batch invite without send emails
	 *
	 * @return	void
	 */
	protected function batch()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'invite_batch' );
		
		\IPS\Session::i()->csrfCheck();

		$form = new \IPS\Helpers\Form( 'form', 'add_invitations' );
		$form->addMessage('is_batch_invitations_desc');

		$form->add( new \IPS\Helpers\Form\Member( 'invite_sender_id', NULL, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Number( 'invite_batch_nr', 5, FALSE, array( 'min' => 1, 'max' => 100 ) ) );

		if( $values = $form->values() )
		{
			$inviter = \IPS\Member::load( $values['invite_sender_id']->member_id );

			for( $i = 1; $i <= $values['invite_batch_nr']; $i++ )
			{
				$newDate = 0;
				if( \IPS\Settings::i()->is_expireinvite > 0 )
				{
					$expire  = new \IPS\DateTime;
					$expire->add( new \DateInterval( 'P' . \IPS\Settings::i()->is_expireinvite . 'D' ) );
					$newDate = $expire->getTimestamp();
				}

				$invitation = new \IPS\invite\Invitation;
				$invitation->sender_id 		 = $values['invite_sender_id']->member_id;
				$invitation->code			 = md5( uniqid( microtime(), true ) );
				$invitation->date 			 = time();
				$invitation->invited_name	 = '';
				$invitation->invited_email	 = '';
				$invitation->status			 = 0;
				$invitation->expiration_date = $newDate;
				$invitation->save();
			}

			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=invite&module=invite&controller=invite" ), 'batch_invite_added' );
		}

		\IPS\Output::i()->output = $form;
	}

	/**
	 * Export invitations to a CSV file
	 *
	 * @return	void
	 */
	protected function download()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'invite_export' );

		$form = new \IPS\Helpers\Form( 'form', 'export_invitations' );
		$form->add( new \IPS\Helpers\Form\Member( 'invite_sender_id', \IPS\Member::loggedIn()->name, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Radio( 'invite_type', 'is_batch', FALSE, array( 'options' => array( 'is_batch' => 'is_batch', 'is_regular' => 'is_regular' ) ) ) );

		if( $values = $form->values() )
		{
			$where[] = array( 'invite_status=?', 0 );
			$where[] = array( 'invite_sender_id=?', $values['invite_sender_id']->member_id );
			if( $values['invite_type'] == 'is_batch' )
			{
				$where[] = array( 'invite_invited_name=?', '' );
			}
			else
			{
				$where[] = array( 'invite_invited_name!=?', '' );
			}

			header( 'Content-Type: text/csv' );
			$headers 	= array();
			$toPut 		= array();
			$fp 		= fopen('php://output', 'w');

			if( $values['invite_type'] == 'is_batch' )
			{
				$headers = array( 'Inviter ID', 'Inviter Name', 'Invitation Date', 'Invitation Code' );
			}
			else
			{
				$headers = array( 'Inviter ID', 'Inviter Name', 'Invitee Name', 'Invitee Email', 'Invitation Date', 'Invitation Code' );
			}

			fputcsv( $fp, $headers );

			foreach( \IPS\Db::i()->select( '*', 'invite_invites', $where ) as $row )
			{
				$toPut[0] = $values['invite_sender_id']->member_id;
				$toPut[1] = $values['invite_sender_id']->name;

				if( $values['invite_type'] == 'is_batch' )
				{
					$toPut[2] = \IPS\DateTime::ts( $row['invite_date'] );
					$toPut[3] = $row['invite_code'];
				}
				else
				{
					$toPut[2] = $row['invite_invited_name'];
					$toPut[3] = $row['invite_invited_email'];
					$toPut[4] = \IPS\DateTime::ts( $row['invite_date'] );
					$toPut[5] = $row['invite_code'];
				}

				fputcsv( $fp, $toPut );
			}

			fclose( $fp );
			die();
		}

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'is_download_invitations' );
		\IPS\Output::i()->output = $form;
	}
}