<?php

namespace IPS\invite\modules\admin\invite;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _members extends \IPS\Dispatcher\Controller
{	
	public static $csrfProtected = TRUE;

	protected function manage()
	{		
		\IPS\Dispatcher::i()->checkAcpPermission( 'members_manage' );

		$table = new \IPS\Helpers\Table\Db( 'invite_members', \IPS\Http\Url::internal( 'app=invite&module=invite&controller=members' ), array( array( 'is_invites_remaining>?', 0 ) ) );
		$table->joins = array(
			array( 'select' => 'm.*', 'from' => array( 'core_members', 'm' ), 'where' => "invite_members.is_member_id=m.member_id" ),
			array( 'select' => 'g.is_unlimited', 'from' => array( 'invite_groups', 'g' ), 'where' => "g.is_id=m.member_group_id" )
		);

		$table->filters = array(
			'invitations_unlimited'	=> 'is_unlimited = 1',
			'invitations_limited'	=> 'is_unlimited = 0',
		);

		$table->include 	= array( 'photo', 'name', 'member_group_id', 'is_invites_remaining' );
		$table->widths 		= array( 'photo' => 7, 'name' => 40, 'member_group_id' => 30, 'is_invites_remaining' => 23 );
		$table->mainColumn 	= 'name';
		$table->noSort		= array( 'photo' );
		$table->classes 	= array( 'ipsDataList', 'ipsDataList_zebra' );
		$table->langPrefix 	= 'is_';

        $table->parsers = array(
            'member_group_id'	=> function( $val, $row )
            {
				try
				{
					return \IPS\Member\Group::load( $val )->formattedName;
				}
				catch( \OutOfRangeException $e )
				{
					return '---';
				}
            },
            'photo' => function( $val, $row )
            {
                return \IPS\Theme::i()->getTemplate( 'global', 'core' )->userPhoto( \IPS\Member::constructFromData( $row ), 'tiny' );
            },
            'is_invites_remaining'	=> function( $val, $row )
            {
				if( $row['is_unlimited'] )
				{
					return '<strong>'.\IPS\Member::loggedIn()->language()->addToStack('invitations_unlimited').'</strong>';
				}
				else
				{
					return $val;
				}
			}
        );

		$table->sortBy  = $table->sortBy ?: 'is_invites_remaining desc desc,name';
        $table->sortDirection = $table->sortDirection ?: 'asc';

		$table->quickSearch = 'name';

		$groups     = array( '' => 'any_group' );
		foreach( \IPS\Member\Group::groups() as $k => $v )
		{
			if( $k != \IPS\Settings::i()->guest_group )
			{
				$groups[ $k ] = $v->name;
			}
		}
		$table->advancedSearch = array(
			'name'					=> \IPS\Helpers\Table\SEARCH_MEMBER,
			'member_group_id'		=> array( \IPS\Helpers\Table\SEARCH_SELECT, array( 'options' => $groups ), function( $val )
			{
				return array( 'member_group_id=? OR FIND_IN_SET( ?, mgroup_others )', $val, $val );
			} ),
			'invites_remaining'		=> \IPS\Helpers\Table\SEARCH_NUMERIC
		);

		if( \IPS\Request::i()->advanced_search_submitted OR \IPS\Request::i()->quicksearch )
		{
			$link = ' <a href="' . \IPS\Http\Url::internal( 'app=invite&module=invite&controller=members' ) . '">' . \IPS\Member::loggedIn()->language()->addToStack('member_view_full_list') . '</a>';
			$table->extraHtml = \IPS\Theme::i()->getTemplate( 'global', 'core' )->message( \IPS\Member::loggedIn()->language()->addToStack( 'member_search_results' ) . $link, 'info', NULL, FALSE );
		}

		\IPS\Output::i()->output = $table;
		\IPS\Output::i()->title  = \IPS\Member::loggedIn()->language()->addToStack('is_invitation_per_member');
	}
}