<?php

namespace IPS\invite\modules\front\invite;

/* To prevent PHP errors (extending class does not exist) revealing path */
if( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _invite extends \IPS\Dispatcher\Controller
{
	public $userData  = NULL;
	public $groupData = NULL;

	public function execute()
	{
		$this->userData	 = \IPS\invite\Member::getMember();
		$this->groupData = \IPS\Data\Store::i()->inviteGroups[ \IPS\Member::loggedIn()->member_group_id ];

		parent::execute();
	}

	protected function manage()
	{
		if( !$this->groupData['is_canvite'] OR $this->userData->revoke_access )
		{
			\IPS\Output::i()->error( 'invite_no_permission', 'INVITE SYSTEM - SEND INVITATION/1', 403, '' );
		}

		if( \IPS\Settings::i()->is_on AND $this->groupData['is_canvite'] AND ( $this->userData->invites_remaining == 0 AND !$this->groupData['is_unlimited'] ) )
		{
			\IPS\Output::i()->error( 'invites_no_remaining_invitesx', 'INVITE SYSTEM - SEND INVITATION/2', 404, '' );
		}

		$form = new \IPS\Helpers\Form( 'form', 'send_invitation' );
		$form->class = 'ipsForm_vertical';
		$form->add( new \IPS\Helpers\Form\Text( 'invite_invited_name', NULL, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Email( 'invite_invited_email', NULL, TRUE, array(), function( $val )
		{
			foreach( \IPS\Login::methods( TRUE ) as $k => $handler )
			{
				if( $handler->emailIsInUse( $val ) === TRUE )
				{
					throw new \LogicException( 'email_already_in_use' );
					break;
				}
			}

			try
			{
				$invite = \IPS\Db::i()->select( '*', 'invite_invites', array( 'invite_invited_email=?', $val ) )->first();
				if( $invite['invite_invited_email'] )
				{
					throw new \LogicException( 'invite_email_already_invited' );
				}
			}
			catch( \UnderflowException $e ){}
		}) );

		if( $values = $form->values() )
		{
			if( \IPS\Settings::i()->is_expireinvite > 0 )
			{
				$expire  = new \IPS\DateTime;
				$expire->add( new \DateInterval( 'P' . \IPS\Settings::i()->is_expireinvite . 'D' ) );
				$newDate = $expire->getTimestamp();
			}
			else
			{
				$newDate = 0;
			}

			$invitationCode = md5( uniqid( microtime(), true ) );
			$invitation 	= new \IPS\invite\Invitation;
			$invitation->sender_id 		 = \IPS\Member::loggedIn()->member_id;
			$invitation->code			 = $invitationCode;
			$invitation->date 			 = time();
			$invitation->invited_name	 = $values['invite_invited_name'];
			$invitation->invited_email	 = $values['invite_invited_email'];
			$invitation->status			 = 0;
			$invitation->expiration_date = $newDate;
			$invitation->save();

			/* Update # of Invites of the sender */
			if( !$this->groupData['is_unlimited'] )
			{
				$this->userData->invites_remaining -= 1;
				$this->userData->save();

				if( $this->userData->invites_remaining == 0 )
				{
					\IPS\Member::clearCreateMenu();
				}
			}

			\IPS\invite\Invite::sendInvitation( $values['invite_invited_email'], $values['invite_invited_name'], $invitationCode, \IPS\Member::loggedIn()->email, \IPS\Member::loggedIn()->name, $newDate );

			$url = \IPS\Http\Url::internal( 'app=core&module=system&controller=settings&area=invitesystem', 'front', 'settings_invitesystem' )->setQueryString( 'area', 'invitesystem' );
			\IPS\Output::i()->redirect( $url, 'invite_sent' );
		}

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'send_new_invitation' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit' )->invitationForm( $form );	
	}

	public function topInviters()
	{
		$limit 		= \intval( ( isset( \IPS\Request::i()->limit ) AND \IPS\Request::i()->limit < 20 ) ? \IPS\Request::i()->limit : 5 );
		$where 		= array( array( 'invite_sender_id > 0' ) );
		$timeframe 	= 'all';

		if( isset( \IPS\Request::i()->time ) AND \IPS\Request::i()->time != 'all' )
		{
			switch ( \IPS\Request::i()->time )
			{
				case 'week':
					$where[] = array( 'invite_conv_date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1W' ) )->getTimestamp() );
					$timeframe = 'week';
				break;
				case 'month':
					$where[] = array( 'invite_conv_date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1M' ) )->getTimestamp() );
					$timeframe = 'month';
				break;
				case 'year':
					$where[] = array( 'invite_conv_date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1Y' ) )->getTimestamp() );
					$timeframe = 'year';
				break;
			}
            $topInviters = iterator_to_array( \IPS\Db::i()->select( 'invite_sender_id as member, count(invite_id) as rep', 'invite_invites', $where, 'rep DESC', $limit, 'member' )->setKeyField('member')->setValueField('rep') );
        }
        else
        {
            $topInviters = iterator_to_array( \IPS\Db::i()->select( 'invite_sender_id as member, count(invite_id) as rep', 'invite_invites', array( 'invite_conv_member_id > 0' ), 'rep DESC', $limit, 'member' )->setKeyField('member')->setValueField('rep') );
        }

		foreach( \IPS\Db::i()->select( '*', 'core_members', \IPS\Db::i()->in( 'member_id', array_keys( $topInviters ) ) ) as $member )
		{
			\IPS\Member::constructFromData( $member );
		}

		$output = \IPS\Theme::i()->getTemplate( 'widgets' )->topInvitersRows( $topInviters, $timeframe, \IPS\Request::i()->orientation );

		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}
	}

	public function payPoints()
	{
		if( !$this->groupData['is_canvite'] OR $this->userData->revoke_access )
		{
			\IPS\Output::i()->error( 'invite_no_permission', 'INVITE SYSTEM - EXCHANGE POINTS/1', 403, '' );
		}

		if( ! \IPS\invite\Invite::canPayPoints() )
		{
			\IPS\Output::i()->error( 'invite_no_permission', 'INVITE SYSTEM - EXCHANGE POINTS/2', 403, '' );
		}

		$form 		 = new \IPS\Helpers\Form( 'exchange', 'is_exchange_points' );
		$form->class = 'ipsForm_vertical';

		$member 		= \IPS\Member::loggedIn();
		$points			= $member->shopPoints();
		$maxInvitations = \intval( $points / \IPS\Settings::i()->is_mshop_paypoints_points );
		$text = \IPS\Member::loggedIn()->language()->addToStack( 'is_exchange_points_blurb', FALSE, array( 'sprintf' => array( \IPS\Settings::i()->is_mshop_paypoints_points, \IPS\Settings::i()->ms_points_name, $points, \IPS\Settings::i()->ms_points_name, $maxInvitations ) ) );

		$form->addMessage( $text, 'ipsMessage ipsMessage_info' );
		$form->add( new \IPS\Helpers\Form\Number( 'is_exchange_number_invitations', 1, FALSE, array( 'min' => 1, 'max' => $maxInvitations ) ) );

		if( $values = $form->values() )
		{
			$total 	= $values['is_exchange_number_invitations'] * \IPS\Settings::i()->is_mshop_paypoints_points;
			$log 	= $member->language()->addToStack( 'is_points_deducted', FALSE, array( 'sprintf' => array( \IPS\Settings::i()->ms_points_name, $values['is_exchange_number_invitations'] ) ) );
			$member->language()->parseOutputForDisplay( $log );
			$this->userData->invites_remaining = $values['is_exchange_number_invitations'];
			$this->userData->save();

			$member->removePoints( $total, $log );

			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=core&module=system&controller=settings&area=invitesystem", 'front', 'settings_invitesystem' ), 'saved' );
		}

		if( !\IPS\Request::i()->isAjax() )
		{
			unset( \IPS\Output::i()->breadcrumb['module'] );
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=system&controller=settings&area=invitesystem', 'front', 'settings_invitesystem' ), \IPS\Member::loggedIn()->language()->addToStack( 'settings' ) );
			\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( '__app_invite' ) . ': ' . \IPS\Member::loggedIn()->language()->addToStack( 'is_exchange_points' ) );

			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'is_exchange_points' );
		}

		\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'forms', 'core' ), 'popupTemplate' ) );
	}

	public function buyInvitation()
	{
		if( !$this->groupData['is_canvite'] OR $this->userData->revoke_access )
		{
			\IPS\Output::i()->error( 'invite_no_permission', 'INVITE SYSTEM - BUY INVITATIONS/1', 403, '' );
		}

		if( ! \IPS\invite\Invite::canBuyInvitations() )
		{
			\IPS\Output::i()->error( 'invite_no_permission', 'INVITE SYSTEM - BUY INVITATIONS/2', 403, '' );
		}

		$form = new \IPS\Helpers\Form( 'buy', 'is_buy_invitation' );
		$form->class = 'ipsForm_vertical';

		$text = \IPS\Member::loggedIn()->language()->addToStack( 'is_exchange_price_blurb', FALSE, array( 'sprintf' => array( \IPS\invite\Invite::price() ) ) );

		$form->addMessage( $text, 'ipsMessage ipsMessage_info' );
		$form->add( new \IPS\Helpers\Form\Number( 'is_purchase_number_invitations', 1, FALSE, array( 'min' => 1 ) ) );

		if( $values = $form->values() )
		{
			$item 			= new \IPS\invite\extensions\nexus\Item\inviteSystem( \IPS\Member::loggedIn()->name, \IPS\invite\Invite::price() );
			$item->id 		= \IPS\Member::loggedIn()->member_id;
			$item->quantity = $values['is_purchase_number_invitations'];

			try
			{
				$item->tax = \IPS\Settings::i()->is_tax ? \IPS\nexus\Tax::load( \IPS\Settings::i()->is_tax ) : NULL;
			}
			catch( \OutOfRangeException $e ) {}

			if( \IPS\Settings::i()->is_payment_method )
			{
				$item->paymentMethodIds = explode( ',', \IPS\Settings::i()->is_payment_method );
			}

			$invoice 				= new \IPS\nexus\Invoice;
			$invoice->currency 		= ( isset( $_SESSION['currency'] ) and \in_array( $_SESSION['currency'], \IPS\nexus\Money::currencies() ) ) ? $_SESSION['currency'] : \IPS\nexus\Customer::loggedIn()->defaultCurrency();
			$invoice->member 		= \IPS\nexus\Customer::loggedIn();
			$invoice->addItem( $item );
			$invoice->return_uri 	= "app=core&module=system&controller=settings&area=invitesystem";
			$invoice->save();

			\IPS\Output::i()->redirect( $invoice->checkoutUrl() );
		}

		if( !\IPS\Request::i()->isAjax() )
		{
			unset( \IPS\Output::i()->breadcrumb['module'] );

			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=system&controller=settings&area=invitesystem', 'front', 'settings_invitesystem' ), \IPS\Member::loggedIn()->language()->addToStack( 'settings' ) );
			\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( '__app_invite' ) . ': ' . \IPS\Member::loggedIn()->language()->addToStack( 'is_buy_invitation' ) );

			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'is_buy_invitation' );
		}

		\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'forms', 'core' ), 'popupTemplate' ) );
	}

	public function sendInvitation()
	{
		if( !$this->groupData['is_canvite'] OR $this->userData->revoke_access )
		{
			\IPS\Output::i()->error( 'invite_no_permission', 'INVITE SYSTEM - SEND BATCH INVITATION/1', 403, '' );
		}

		if( \IPS\Settings::i()->is_on AND $this->groupData['is_canvite'] AND ( $this->userData->invites_remaining == 0 AND !$this->groupData['is_unlimited'] ) )
		{
			\IPS\Output::i()->error( 'invites_no_remaining_invitesx', 'INVITE SYSTEM - SEND BATCH INVITATION/2', 404, '' );
		}

		try
		{
			$invitation = \IPS\invite\Invitation::load( \IPS\Request::i()->id );
		}
		catch( \OutOfRangeException $ex )
		{
			\IPS\Output::i()->error( 'invite_does_not_exist', 'INVITE SYSTEM - SEND BATCH INVITATION/3', 404, '' );
		}

		$form = new \IPS\Helpers\Form( 'form', 'send_invitation' );
		$form->class = 'ipsForm_vertical';
		$form->add( new \IPS\Helpers\Form\Text( 'invite_invited_name', NULL, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Email( 'invite_invited_email', NULL, TRUE, array(), function( $val )
		{
			foreach( \IPS\Login::methods( TRUE ) as $k => $handler )
			{
				if( $handler->emailIsInUse( $val ) === TRUE )
				{
					throw new \LogicException( 'email_already_in_use' );
					break;
				}
			}

			try
			{
				$invite = \IPS\Db::i()->select( '*', 'invite_invites', array( 'invite_invited_email=?', $val ) )->first();
				if( $invite['invite_invited_email'] )
				{
					throw new \LogicException( 'invite_email_already_invited' );
				}
			}
			catch( \UnderflowException $e ){}
		}) );

		if( $values = $form->values() )
		{
			if( \IPS\Settings::i()->is_expireinvite > 0 )
			{
				$expire  = new \IPS\DateTime;
				$expire->add( new \DateInterval( 'P' . \IPS\Settings::i()->is_expireinvite . 'D' ) );
				$newDate = $expire->getTimestamp();
			}
			else
			{
				$newDate = 0;
			}

			$invitationCode 			 = $invitation->code;
			$invitation->date 			 = time();
			$invitation->invited_name	 = $values['invite_invited_name'];
			$invitation->invited_email	 = $values['invite_invited_email'];
			$invitation->status			 = 0;
			$invitation->expiration_date = $newDate;
			$invitation->save();

			/* Update # of Invites of the sender */
			if( !$this->groupData['is_unlimited'] )
			{
				$this->userData->invites_remaining -= 1;
				$this->userData->save();

				if( $this->userData->invites_remaining == 0 )
				{
					\IPS\Member::clearCreateMenu();
				}
			}

			\IPS\invite\Invite::sendInvitation( $values['invite_invited_email'], $values['invite_invited_name'], $invitationCode, \IPS\Member::loggedIn()->email, \IPS\Member::loggedIn()->name, $newDate );

			$url = \IPS\Http\Url::internal( 'app=core&module=system&controller=settings&area=invitesystem', 'front', 'settings_invitesystem' )->setQueryString( 'area', 'invitesystem' );
			\IPS\Output::i()->redirect( $url, 'invite_sent' );
		}

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'send_new_invitation' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit' )->invitationForm( $form );	
	}

	public function resendInvitation()
	{
		\IPS\Session::i()->csrfCheck();

		try
		{
			$invitation = \IPS\invite\Invitation::load( \IPS\Request::i()->id );
		}
		catch( \Exception $e )
		{
			\IPS\Output::i()->error( 'invite_does_not_exist', 'INVITE SYSTEM - RESEND INVITATION/1', 404, '' );
		}

		if( $invitation->status != 0 )
		{
			\IPS\Output::i()->error( 'invite_not_pending', 'INVITE SYSTEM - RESEND INVITATION/2', 403, '' );
		}

		if( $invitation->sender_id != \IPS\Member::loggedIn()->member_id )
		{
			\IPS\Output::i()->error( 'invite_not_mine', 'INVITE SYSTEM - RESEND INVITATION/3', 403, '' );
		}

		\IPS\invite\Invite::sendInvitation( $invitation->invited_email, $invitation->invited_name, $invitation->code, \IPS\Member::loggedIn()->email, \IPS\Member::loggedIn()->name, $invitation->expiration_date );

		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=core&module=system&controller=settings&area=invitesystem", 'front', 'settings_invitesystem' ), 'invite_sent' );
	}

	public function deleteInvitation()
	{
		\IPS\Session::i()->csrfCheck();

		try
		{
			$invitation = \IPS\invite\Invitation::load( \IPS\Request::i()->id );
		}
		catch( \Exception $e )
		{
			\IPS\Output::i()->error( 'invite_does_not_exist', 'INVITE SYSTEM - DELETE INVITATION/1', 404, '' );
		}

		if( $invitation->status != 0 )
		{
			\IPS\Output::i()->error( 'invite_not_pending', 'INVITE SYSTEM - DELETE INVITATION/2', 403, '' );
		}

		if( $invitation->sender_id != \IPS\Member::loggedIn()->member_id )
		{
			\IPS\Output::i()->error( 'invite_not_mine', 'INVITE SYSTEM - DELETE INVITATION/3', 403, '' );
		}

		$invitation->delete();

		if( !$this->groupData['is_unlimited'] )
		{
			$this->userData->invites_remaining += 1;
			$this->userData->save();
		}

		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=core&module=system&controller=settings&area=invitesystem", 'front', 'settings_invitesystem' ), 'invite_sent' );
	}

    public function getConvertedMembers()
    {
		$converted = array();
		if( \IPS\Settings::i()->is_on AND $this->groupData['is_canvite'] )
		{
			foreach( \IPS\Db::i()->select( '*', 'invite_invites', array( "invite_sender_id=? AND invite_status=?", \IPS\Member::loggedIn()->member_id, 1 ), NULL, array( 0, 10 ) )->join( 'core_members', 'invite_invites.invite_conv_member_id=core_members.member_id' ) as $member )
			{
				$converted[ $member['invite_conv_member_id'] ] = $member;
			}
		}

		\IPS\Output::i()->json( array( 'html' => \IPS\Theme::i()->getTemplate( 'global', 'invite' )->invitesAjax( $converted ) ) );
	}
}