<?php

namespace IPS\invite;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _Invite
{
	/**
	 * Send the invitation via email
	 *
	 * @return	string
	 */
	public static function sendInvitation( $sendToEmail, $sendToName, $invitationCode, $fromEmail, $fromName, $date )
	{
		$inviter		= \IPS\Settings::i()->is_show_inviter_email ? \IPS\Member::loggedIn()->name : '';
		$mail 			= \IPS\Email::buildFromTemplate( 'invite', 'invite_system', array( $sendToName, $invitationCode, $date, $sendToEmail, $inviter ), \IPS\Email::TYPE_TRANSACTIONAL );

		if( \IPS\Settings::i()->mail_method != 'smtp' )
		{
			$mail->from 	= $fromEmail;
			$mail->fromName = $fromName;
		}

		$mail->send( $sendToEmail );
	}

	public static function canUsePoints()
	{
		if( \IPS\Application::appIsEnabled( 'membersshop' ) AND \IPS\Application::load( 'membersshop' )->long_version >= 11003 )
		{
			return TRUE;
		}

		return FALSE;
	}

	public static function canPayPoints()
	{
		$member = \IPS\Member::loggedIn();
		if( static::canUsePoints() AND \IPS\Settings::i()->is_mshop AND \IPS\Settings::i()->is_mshop_paypoints AND \IPS\Settings::i()->is_mshop_paypoints_points > 0 AND $member->shopPoints() >= \IPS\Settings::i()->is_mshop_paypoints_points )
		{
			return TRUE;
		}

		return FALSE;
	}

	public static function canUseCommerce()
	{
		if( \IPS\Application::appIsEnabled( 'nexus' ) )
		{
			return TRUE;
		}

		return FALSE;
	}

	public static function canBuyInvitations()
	{
		if( static::canUseCommerce() AND \IPS\Settings::i()->is_commerce AND \IPS\Settings::i()->is_commerce_cost AND \count( \IPS\nexus\Gateway::roots() ) )
		{
			return TRUE;
		}

		return FALSE;
	}

	public static function price()
	{
		return static::_price( \IPS\Settings::i()->is_commerce_cost );
	}

	public static function _price( $cost )
	{
		if ( $cost )
		{
			$currency = ( isset( $_SESSION['currency'] ) and \in_array( $_SESSION['currency'], \IPS\nexus\Money::currencies() ) ) ? $_SESSION['currency'] : \IPS\nexus\Customer::loggedIn()->defaultCurrency();
			
			/* If $cost is an empty JSON array, the conditional will evaluate false thus resulting in [] being passed to \IPS\nexus\Money (which will fail). */
			$costs = json_decode( $cost, TRUE );
			if ( \is_array( $costs ) )
			{
				if ( isset( $costs[ $currency ]['amount'] ) and $costs[ $currency ]['amount'] )
				{
					return new \IPS\nexus\Money( $costs[ $currency ]['amount'], $currency );
				}
			}
			else
			{
				return new \IPS\nexus\Money( $cost, $currency );
			}
		}
		
		return NULL;
	}

	public function dummy()
	{
		//nothing here
	}
}